/* 
* A primeira seção da especificação vai até o primeiro %%,
* e consiste de código Java que é copiado ao pé da letra
*
*/

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.io.IOException;
import java.util.Stack;

%%

/*
* A segunda seção vai até o próximo %%, e são diversos parâmetros
* de configuração, além de código Java copiado para o corpo da
* classe do analisador léxico
*
*/

%class Scanner          // nome da classe do analisador
%public                 // classe deve ser pública
%line                   // guarde número da linha em yyline
%function token         // nome do método que vai fornecer um token
%type Token             // classe usado para tokens

// Código Java entre %{ e %} é copiado pro corpo da classe
// do analisador
%{ 
	// Níveis de identação
	Stack<Integer> levels;

	public Scanner() { }

	public void input(String input) {
        // Inicializa pilha de identação
	    levels = new Stack<Integer>();
        levels.push(0);
	    // inicializa entrada pro analisador
		yyreset(new StringReader(input));
	}
	
	public List<Token> tokens() throws IOException {
		List<Token> tokens = new ArrayList<Token>();
		Token tok = token();
		while(tok.tipo != Token.EOF) {
			tokens.add(tok);
			tok = token();
		}
		tokens.add(tok);
		return tokens;
	}

%}

%init{
    // Inicializa pilha de identação
	levels = new Stack<Integer>();
    levels.push(0);
%init}

%%

/*
* A última seção contém as regras léxicas, cada regra é um
* par com uma expressão regular e um trecho de código Java
* entre { e }.
*
*/

// Identação, o programa deve começar com um \n extra
// só linhas que não estão em branco são levadas em conta
\n[ ]*.               { 
                        yypushback(1); // volta o primeiro char do prox. token
 						int level = yytext().length()-1;
						// nível corrente
 						int curr = levels.peek();
 						if(level > curr) {
 						  // indenta
 						  levels.push(level);
 						  return new Token(Token.BEGIN, yyline);
 						} else if(level < curr) {
						  levels.pop();
						  // vai rodar o scanner de novo nessa
						  // indentação, para ver se precisa
						  // gerar mais de um }
						  yypushback(level+1);
						  return new Token(Token.END, yyline);
 						} else {
 						  // mesmo nível, não faz nada!
 						}
                      } 

// Espaços em branco
[ \n\r\t\f]  { }

// Comentários
[{]~[}]      { }

// Palavras reservadas
[rR][eE][aA][dD]      { return new Token(Token.READ, yyline); }
[wW][rR][iI][tT][eE]  { return new Token(Token.WRITE, yyline); }
[iI][fF]              { return new Token(Token.IF, yyline); }
[eE][lL][sS][eE]      { return new Token(Token.ELSE, yyline); }
[eE][lL][iI][fF]      { return new Token(Token.ELIF, yyline); }
[wW][hH][iI][lL][eE]  { return new Token(Token.WHILE, yyline); }
[iI][nN][tT]          { return new Token(Token.INT, yyline); }
[bB][oO][oO][lL]      { return new Token(Token.BOOL, yyline); }
[vV][oO][iI][dD]      { return new Token(Token.VOID, yyline); }
[aA][nN][dD]          { return new Token(Token.AND, yyline); }
[oO][rR]              { return new Token(Token.OR, yyline); }
[nN][oO][tT]          { return new Token(Token.NOT, yyline); }
[pP][aA][sS][sS]      { return new Token(Token.PASS, yyline); }
[tT][rR][uU][eE]      { return new Token(Token.TRUE, yyline); }
[fF][aA][lL][sS][eE]  { return new Token(Token.FALSE, yyline); }

// Identificadores e numerais devem ser retornados com
// return new Token(Token.ID, yytext(), yyline)
// e return new Token(Token.NUM, yytext(), yyline)
[a-zA-Z][a-zA-Z0-9_]* { return new Token(Token.ID, yytext(), yyline); }
[0-9]+                { return new Token(Token.NUM, yytext(), yyline); }

// Operadores e pontuação
[:][=]                    { return new Token(Token.ATRIB, yytext(), yyline); }
[)]|[(]|[,]|[=]|[<]|[+]|[\-]|[*]|[/]|[:]  { return new Token((int)(yytext().charAt(0)), yyline); }

// Regra para EOF
<<EOF>>      { return new Token(Token.EOF, yyline); }

// Erros léxicos 
.            { throw new RuntimeException("erro léxico, linha: " +  (yyline+1) + ", char: " + yytext()); }

